@@ -15,13 +15,17 @@ TESTS="$*"
 #
 TMP1=.tmp1
 TMP2=.tmp2
+TMP3=.tmp3
+TMP4=.tmp4
 
 if which mktemp >/dev/null 2>&1; then
 	TMP1=`mktemp`
 	TMP2=`mktemp`
+	TMP3=`mktemp`
+	TMP4=`mktemp`
 fi
 
-export PROG TMP1 TMP2	# variables used by test scripts
+export PROG TMP1 TMP2 TMP3 TMP4	# variables used by test scripts
 
 FAIL=0
 
@@ -41,7 +45,7 @@ for SCRIPT in $TESTS; do
 	test $STATUS -eq 1 && echo "FAILED" || echo "OK"
 done
 
-rm -f $TMP1 $TMP2
+rm -f $TMP1 $TMP2 $TMP3 $TMP4
 
 exit $FAIL
 
