@@ -2,11 +2,10 @@
  * Main program entry point - read the command line options, then perform
  * the appropriate actions.
  *
- * Copyright 2012 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2013 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
-#include "options.h"
-#include "pv.h"
+#include "pv-internal.h"
 
 #define _GNU_SOURCE 1
 #include <limits.h>
@@ -15,19 +14,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/time.h>
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#define RATE_GRANULARITY	100000	    /* usec between -L rate chunks */
-
-extern struct timeval pv_sig_toffset;
-extern sig_atomic_t pv_sig_newsize;
-extern sig_atomic_t pv_sig_abort;
 
 
 /*
@@ -54,14 +41,14 @@ static void pv_timeval_add_usec(struct timeval *val, long usec)
  *
  * Returns nonzero on error.
  */
-int pv_main_loop(opts_t opts)
+int pv_main_loop(pvstate_t state)
 {
 	long written, lineswritten;
 	long long total_written, since_last, cansend;
 	long double target;
 	int eof_in, eof_out, final_update;
 	struct timeval start_time, next_update, next_ratecheck, cur_time;
-	struct timeval init_time;
+	struct timeval init_time, next_remotecheck;
 	long double elapsed;
 	struct stat64 sb;
 	int fd, n;
@@ -83,7 +70,7 @@ int pv_main_loop(opts_t opts)
 
 	fd = -1;
 
-	pv_crs_init(opts);
+	pv_crs_init(state);
 
 	eof_in = 0;
 	eof_out = 0;
@@ -96,42 +83,65 @@ int pv_main_loop(opts_t opts)
 	next_update.tv_sec = start_time.tv_sec;
 	next_update.tv_usec = start_time.tv_usec;
 	pv_timeval_add_usec(&next_update,
-			    (long) (1000000.0 * opts->interval));
+			    (long) (1000000.0 * state->interval));
 
 	next_ratecheck.tv_sec = start_time.tv_sec;
 	next_ratecheck.tv_usec = start_time.tv_usec;
+	next_remotecheck.tv_sec = start_time.tv_sec;
+	next_remotecheck.tv_usec = start_time.tv_usec;
 
 	target = 0;
-	cansend = 0;
 	final_update = 0;
 	n = 0;
 
-	fd = pv_next_file(opts, n, -1);
+	fd = pv_next_file(state, n, -1);
 	if (fd < 0) {
-		return opts->exit_status;
+		if (state->cursor)
+			pv_crs_fini(state);
+		return state->exit_status;
 	}
 
-	if (fstat64(fd, &sb) == 0) {
-		pv_set_buffer_size(sb.st_blksize * 32, 0);
+	/*
+	 * Set target buffer size if the initial file's block size can be
+	 * read and we weren't given a target buffer size.
+	 */
+	if (0 == fstat64(fd, &sb)) {
+		unsigned long long sz;
+		sz = sb.st_blksize * 32;
+		if (sz > BUFFER_SIZE_MAX)
+			sz = BUFFER_SIZE_MAX;
+		state->target_buffer_size = sz;
 	}
 
-	if (opts->buffer_size > 0) {
-		pv_set_buffer_size(opts->buffer_size, 1);
-	}
+	if (0 == state->target_buffer_size)
+		state->target_buffer_size = BUFFER_SIZE;
 
 	while ((!(eof_in && eof_out)) || (!final_update)) {
 
-		if (pv_sig_abort)
+		cansend = 0;
+
+		/*
+		 * Check for remote messages from -R every short while
+		 */
+		if ((cur_time.tv_sec > next_remotecheck.tv_sec)
+		    || (cur_time.tv_sec == next_remotecheck.tv_sec
+			&& cur_time.tv_usec >= next_remotecheck.tv_usec)) {
+			pv_remote_check(state);
+			pv_timeval_add_usec(&next_remotecheck,
+					    REMOTE_INTERVAL);
+		}
+
+		if (state->pv_sig_abort)
 			break;
 
-		if (opts->rate_limit > 0) {
+		if (state->rate_limit > 0) {
 			gettimeofday(&cur_time, NULL);
 			if ((cur_time.tv_sec > next_ratecheck.tv_sec)
 			    || (cur_time.tv_sec == next_ratecheck.tv_sec
 				&& cur_time.tv_usec >=
 				next_ratecheck.tv_usec)) {
 				target +=
-				    ((long double) (opts->rate_limit)) /
+				    ((long double) (state->rate_limit)) /
 				    (long double) (1000000 /
 						   RATE_GRANULARITY);
 				pv_timeval_add_usec(&next_ratecheck,
@@ -140,29 +150,57 @@ int pv_main_loop(opts_t opts)
 			cansend = target;
 		}
 
-		written =
-		    pv_transfer(opts, fd, &eof_in, &eof_out, cansend,
-				&lineswritten);
-		if (written < 0)
-			return opts->exit_status;
+		/*
+		 * If we have to stop at "size" bytes, make sure we don't
+		 * try to write more than we're allowed to.
+		 */
+		if ((0 < state->size) && (state->stop_at_size)) {
+			if ((state->size < (total_written + cansend))
+			    || ((0 == cansend)
+				&& (0 == state->rate_limit))) {
+				cansend = state->size - total_written;
+				if (0 >= cansend) {
+					eof_in = 1;
+					eof_out = 1;
+				}
+			}
+		}
 
-		if (opts->linemode) {
+		if ((0 < state->size) && (state->stop_at_size)
+		    && (0 >= cansend) && eof_in && eof_out) {
+			written = 0;
+		} else {
+			written =
+			    pv_transfer(state, fd, &eof_in, &eof_out,
+					cansend, &lineswritten);
+		}
+
+		if (written < 0) {
+			if (state->cursor)
+				pv_crs_fini(state);
+			return state->exit_status;
+		}
+
+		if (state->linemode) {
 			since_last += lineswritten;
 			total_written += lineswritten;
-			if (opts->rate_limit > 0)
+			if (state->rate_limit > 0)
 				target -= lineswritten;
 		} else {
 			since_last += written;
 			total_written += written;
-			if (opts->rate_limit > 0)
+			if (state->rate_limit > 0)
 				target -= written;
 		}
 
-		if (eof_in && eof_out && n < (opts->argc - 1)) {
+		if (eof_in && eof_out && n < (state->input_file_count - 1)) {
 			n++;
-			fd = pv_next_file(opts, n, fd);
-			if (fd < 0)
-				return opts->exit_status;
+			fd = pv_next_file(state, n, fd);
+			if (fd < 0) {
+				if (state->cursor)
+					pv_crs_fini(state);
+				return state->exit_status;
+			}
 			eof_in = 0;
 			eof_out = 0;
 		}
@@ -174,7 +212,7 @@ int pv_main_loop(opts_t opts)
 			next_update.tv_sec = cur_time.tv_sec - 1;
 		}
 
-		if (opts->no_op)
+		if (state->no_op)
 			continue;
 
 		/*
@@ -183,8 +221,8 @@ int pv_main_loop(opts_t opts)
 		 * we then count time as if we started when the first byte
 		 * was received.
 		 */
-		if (opts->wait) {
-			if (opts->linemode) {
+		if (state->wait) {
+			if (state->linemode) {
 				if (lineswritten < 1)
 					continue;
 			} else {
@@ -192,7 +230,7 @@ int pv_main_loop(opts_t opts)
 					continue;
 			}
 
-			opts->wait = 0;
+			state->wait = 0;
 
 			/*
 			 * Reset the timer offset counter now that data
@@ -206,15 +244,15 @@ int pv_main_loop(opts_t opts)
 			 */
 			pv_sig_nopause();
 			gettimeofday(&start_time, NULL);
-			pv_sig_toffset.tv_sec = 0;
-			pv_sig_toffset.tv_usec = 0;
+			state->pv_sig_toffset.tv_sec = 0;
+			state->pv_sig_toffset.tv_usec = 0;
 			pv_sig_allowpause();
 
 			next_update.tv_sec = start_time.tv_sec;
 			next_update.tv_usec = start_time.tv_usec;
 			pv_timeval_add_usec(&next_update,
 					    (long) (1000000.0 *
-						    opts->interval));
+						    state->interval));
 		}
 
 		if ((cur_time.tv_sec < next_update.tv_sec)
@@ -224,7 +262,7 @@ int pv_main_loop(opts_t opts)
 		}
 
 		pv_timeval_add_usec(&next_update,
-				    (long) (1000000.0 * opts->interval));
+				    (long) (1000000.0 * state->interval));
 
 		if (next_update.tv_sec < cur_time.tv_sec) {
 			next_update.tv_sec = cur_time.tv_sec;
@@ -235,9 +273,9 @@ int pv_main_loop(opts_t opts)
 		}
 
 		init_time.tv_sec =
-		    start_time.tv_sec + pv_sig_toffset.tv_sec;
+		    start_time.tv_sec + state->pv_sig_toffset.tv_sec;
 		init_time.tv_usec =
-		    start_time.tv_usec + pv_sig_toffset.tv_usec;
+		    start_time.tv_usec + state->pv_sig_toffset.tv_usec;
 		if (init_time.tv_usec >= 1000000) {
 			init_time.tv_sec++;
 			init_time.tv_usec -= 1000000;
@@ -254,34 +292,27 @@ int pv_main_loop(opts_t opts)
 		if (final_update)
 			since_last = -1;
 
-		if (pv_sig_newsize) {
-			pv_sig_newsize = 0;
-			pv_screensize(opts);
+		if (state->pv_sig_newsize) {
+			state->pv_sig_newsize = 0;
+			pv_screensize(&(state->width), &(state->height));
 		}
 
-		pv_display(opts, elapsed, since_last, total_written);
+		pv_display(state, elapsed, since_last, total_written);
 
 		since_last = 0;
 	}
 
-	if (opts->cursor) {
-		pv_crs_fini(opts);
+	if (state->cursor) {
+		pv_crs_fini(state);
 	} else {
-		if ((!opts->numeric) && (!opts->no_op))
+		if ((!state->numeric) && (!state->no_op))
 			write(STDERR_FILENO, "\n", 1);
 	}
 
-	/*
-	 * Free up the buffers used by the display and data transfer
-	 * routines.
-	 */
-	pv_display(0, 0, 0, 0);
-	pv_transfer(0, -1, 0, 0, 0, NULL);
+	if (state->pv_sig_abort)
+		state->exit_status |= 32;
 
-	if (pv_sig_abort)
-		opts->exit_status |= 32;
-
-	return opts->exit_status;
+	return state->exit_status;
 }
 
 /* EOF */
